from collections import namedtuple

from box import Box
from enum import Enum

Definition = namedtuple('Definition', ('name', 'type', 'children'))


class Profile(Box):
    def __init__(self, *args, **kwargs):
        kwargs['frozen_box'] =True
        super(Profile, self).__init__(*args, **kwargs)


class Return(Enum):
    SUCCESS = 0b0000
    ERROR = 0b1000
    EXIT = 0b0001
    WRITE = 0b0010
    READ = 0b0100




from logging import Logger
from typing import List, Text
from datetime import datetime
from uuid import uuid4

from waterch.tasker import Definition, Profile, Return, value
from waterch.tasker.storage import Storage
from waterch.tasker.tasks._base import Task


class GenerateIdTask(Task):
    def generate_datetime(self) -> Text:
        return datetime.now().strftime('%Y%m%d%H%M%S')

    def generate_uuid(self) -> Text:
        return uuid4().hex.replace('-', '')

    def invoke(self, profile: Profile, shared: Storage, logger: Logger) -> Return:
        def generate(name: Text) -> Text:
            if name == 'datetime':
                return self.generate_datetime()
            elif name == 'uuid':
                return self.generate_uuid()
            elif name == 'label':
                return profile.label
            else:
                return ''

        shared['id'] = profile.by.join(filter(lambda t: t != '', map(generate, profile.join)))
        logger.debug(f'ID: {shared["id"]}')
        return Return.SUCCESS.value

    @classmethod
    def require(cls) -> List[Text]:
        return []

    @classmethod
    def provide(cls) -> List[Text]:
        return ['id']

    @classmethod
    def define(cls) -> List[Definition]:
        return [
            value('by', str),
            value('join', list, [str]),
            value('label', str)
        ]



__all__ = [
    'Task',
]

from abc import ABCMeta, abstractmethod
from logging import Logger
from typing import List, Text

from waterch.tasker import Profile, Definition, Return, value
from waterch.tasker.mixin import ProfileMixin
from waterch.tasker.storage import Storage


class Task(ProfileMixin, metaclass=ABCMeta):
    @abstractmethod
    def invoke(self, profile: Profile, shared: Storage, logger: Logger) -> int:
        raise NotImplementedError('Please implement the task process.')

    @classmethod
    @abstractmethod
    def require(cls) -> List[Text]:
        raise NotImplementedError('Please define required keys.')

    @classmethod
    @abstractmethod
    def provide(cls) -> List[Text]:
        raise NotImplementedError('Please define provided keys.')



__all__ = [
    'Storage',
    'DictStorage',
]

from abc import ABCMeta, abstractmethod
from sys import stderr
from typing import Text, Any
from collections import OrderedDict


class Storage(metaclass=ABCMeta):
    def __init__(self, **kwargs):
        pass

    @abstractmethod
    def __getitem__(self, item):
        raise NotImplementedError('Please implement __getitem__ function instead of using abstract storage.')

    @abstractmethod
    def __setitem__(self, key, value):
        raise NotImplementedError('Please implement __setitem__ function instead of using abstract storage.')

    @abstractmethod
    def __delitem__(self, key):
        raise NotImplementedError('Please implement __delitem__ function instead of using abstract storage.')

    @abstractmethod
    def load(self):
        raise NotImplementedError('Please implement load function instead of using abstract storage.')

    @abstractmethod
    def dump(self):
        raise NotImplementedError('Please implement dump function instead of using abstract storage.')


class DictStorage(Storage):
    __slots__ = ['_data']

    def __init__(self, **kwargs):
        super(DictStorage, self).__init__(**kwargs)
        self._data = OrderedDict()

    def __getitem__(self, item: Text) -> Any:
        return self._data.__getitem__(item)

    def __setitem__(self, key: Text, value: Any):
        return self._data.__setitem__(key, value)

    def __delitem__(self, key: Text):
        return self._data.__delitem__(key)

    def load(self):
        print('Function load is invalid in DictStorage.', file=stderr)

    def dump(self):
        print('Function dump is invalid in DictStorage.', file=stderr)
